#include "mainwindow.h"

MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent)
{
    menuBar = new QMenuBar();

    //3.2.1 File Menu
    fileMenu = new QMenu("&File");

    exitAction = new QAction("E&xit",fileMenu);
//  exitAction->setShortcut(QKeySequence::Quit);//original code from assignment. Ctrl+q is not supported on windows.
    exitAction->setShortcut(QKeySequence(tr("Ctrl+Q")));//workaround for windows.
    connect(exitAction , SIGNAL(triggered ( ) ) ,    this , SLOT(close ( ) ) ) ;

    fileMenu->addAction(exitAction );
    menuBar->addMenu(fileMenu );


    //3.2.2 Shading Menu
    shadingMenu = new QMenu("Shading");

    shadeWireframeAction = new QAction("Wireframe",shadingMenu);
    shadeFlatAction = new QAction("Flat",shadingMenu);
    shadeGouraudAction = new QAction("Gouraud",shadingMenu);
    shadePhongAction = new QAction("Phong",shadingMenu);

    shadeWireframeAction->setShortcut(QKeySequence(tr("Ctrl+1")));
    shadeFlatAction->setShortcut(QKeySequence(tr("Ctrl+2")));
    shadeGouraudAction->setShortcut(QKeySequence(tr("Ctrl+3")));
    shadePhongAction->setShortcut(QKeySequence(tr("Ctrl+4")));

    shadeWireframeAction->setCheckable(true);
    shadeFlatAction->setCheckable(true);
    shadeGouraudAction->setCheckable(true);
    shadePhongAction->setCheckable(true);

    shadeWireframeAction->setIcon(QIcon(":/img/wireframe.png"));
    shadeFlatAction->setIcon(QIcon(":/img/flat.png"));
    shadeGouraudAction->setIcon(QIcon(":/img/gouraud.png"));
    shadePhongAction->setIcon(QIcon(":/img/phong.png"));

    shaderGroup = new QActionGroup(shadingMenu);
    shaderGroup->addAction(shadeWireframeAction);
    shaderGroup->addAction(shadeFlatAction);
    shaderGroup->addAction(shadeGouraudAction);
    shaderGroup->addAction(shadePhongAction);

    shadingMenu->addAction(shadeWireframeAction);
    shadingMenu->addAction(shadeFlatAction);
    shadingMenu->addAction(shadeGouraudAction);
    shadingMenu->addAction(shadePhongAction);

    shadeFlatAction->setChecked(true);

    menuBar->addMenu(shadingMenu);

    setMenuBar(menuBar);

    //3.2.3 About Message Box
    aboutAction = new QAction("About",menuBar);
    connect(aboutAction, SIGNAL(triggered()), this,SLOT(showAboutBox()));
    menuBar->addAction(aboutAction);

    //3.3 Toolbar
    toolBar = new QToolBar("ToolBar",this);
    toolBar->addAction(shadeWireframeAction);
    toolBar->addAction(shadeFlatAction);
    toolBar->addAction(shadeGouraudAction);
    toolBar->addAction(shadePhongAction);
    addToolBar( toolBar);

    //3.4 Status Bar
    statusBar = new QStatusBar(this);
    statusBar->showMessage("test status message");
    setStatusBar(statusBar);

    //4.0 Widget
    myGLWidget = new MyGLWidget(this);
    setCentralWidget(myGLWidget);

    //4.1.1 connect shading actions to slot
    connect(shadeWireframeAction,SIGNAL(triggered(bool)),myGLWidget,SLOT(shadeWireframeSlot()));
    connect(shadeFlatAction,SIGNAL(triggered(bool)),myGLWidget,SLOT(shadeFlatSlot()));
    connect(shadeGouraudAction,SIGNAL(triggered(bool)),myGLWidget,SLOT(shadeGouraudSlot()));
    connect(shadePhongAction,SIGNAL(triggered(bool)),myGLWidget,SLOT(shadePhongSlot()));

    //tesselation slider
    tesselationSlider = new QSlider(Qt::Horizontal,this);
    tesselationSlider->setTickPosition(QSlider::TicksBelow);
    tesselationSlider->setTickInterval(2);
    tesselationSlider->setSizePolicy(QSizePolicy::Fixed,QSizePolicy::Fixed);
    tesselationSlider->setMinimum(1);
    tesselationSlider->setMaximum(16);
    tesselationSlider->setValue(1);
    connect(tesselationSlider,SIGNAL(valueChanged(int)),myGLWidget,SLOT(setTessellation(int)));
    toolBar->addWidget(tesselationSlider);

    //reset Camera button
    resetCameraAction = new QAction("Reset Camera",toolBar);
    resetCameraAction->setIcon(QIcon(":/img/cam_home.png"));
    connect(resetCameraAction,SIGNAL(triggered(bool)),myGLWidget,SLOT(resetCamera()));
    toolBar->addAction(resetCameraAction);

}

MainWindow::~MainWindow()
{

}

//3.2.3 about message box
void MainWindow::showAboutBox(){
    QMessageBox msgBox;
    msgBox.setWindowTitle("About Hello Cube!");
    msgBox.setText("Written by Alisa");
    msgBox.exec();
}
