#ifndef GLWIDGET_H
#define GLWIDGET_H

#include <QGLWidget>
#include <gl/GLU.h>
#include <QDebug>
#include <QWheelEvent>
#include <QQuaternion>
#include <QMatrix4x4>
#include <math.h>
#include <QVector3D>
#include <QGLShaderProgram>
#include <QGLShader>

class MyGLWidget : public QGLWidget
{
    Q_OBJECT //from HelloGL Example

   public:
       MyGLWidget(QWidget *parent = 0);
       ~MyGLWidget();

       //From HelloGL example.
       QSize minimumSizeHint() const;//do I need this?
       QSize sizeHint() const;//do I need this?

protected:
       //4.1 Core Functionality
    void initializeGL();
    void paintGL();
    void resizeGL(int width, int height);
    void mousePressEvent(QMouseEvent *event);
    void mouseMoveEvent(QMouseEvent *event);
    void wheelEvent(QWheelEvent *event);

private:
    //tesselation slider
    int tesselation;

    //zoom camera
    const double cameraZoomDefault = -5;
    double cameraZoom;

    //translate camera
    QPoint clickStartPosition;
    bool rightButtonPressed;
    double tx, ty;

    //rotate cube
    bool leftButtonPressed;
    QQuaternion qubeRotationQuaternion;
    QPoint screenCenter;//Center of screen in pixel coordinates

    //Phong Shader
    QGLShaderProgram *phongShader;


public slots:
    //4.1.1 slots for shading modes
    void shadeWireframeSlot();
    void shadeFlatSlot();
    void shadeGouraudSlot();
    void shadePhongSlot();
    void setTessellation(int t);
    void resetCamera();
};

#endif // GLWIDGET_H
